from typing import TypeAlias, Callable, TypeVar, Any
from .spacy_types import Doc, Span, Token
from .sklearn_types import csc_matrix, csr_matrix
from .numpy_types import ndarray, NDArray, str_
from .pandas_types import Index

# TextProcessor
TokenIterator: TypeAlias = Doc | Span | list[Token]
POSIterator: TypeAlias = list[tuple[Token, str]]
SentenceIterator: TypeAlias = list[Span] | list[list[Token]] | list[POSIterator]
TextIterator: TypeAlias = list[str] | list[tuple[str, str]] | list[list[tuple[str, str]]]

Sentence: TypeAlias = Span | list[Token] | POSIterator | list[tuple[str, str]]
Sentences: TypeAlias =  list[Sentence]
POSTags: TypeAlias = list[tuple[Token | str, str]]
Function = TypeVar("Function", bound=Callable[..., object])

# TargetCollocations
Collocate = tuple[str, tuple[str, str]] | tuple[str, str]

# Matrices
Matrix: TypeAlias = csr_matrix | csc_matrix | ndarray
Labels: TypeAlias = NDArray[str_] | Index | list[str] | None

#Workflow
MethodSpec: TypeAlias = dict[str, dict[str, Any]]