from .workflow_tools import WorkflowBuilder, WorkflowManager

from typing import Any, Callable

class PipelineBuilder:
    """
    Automates the execution of a sequence of workflows on a WorkflowBuilder or Callable subclass.

    :param workflows: A subclass of WorkflowBuilder representing the workflow to execute or a function to hook in the pipeline.
    """

    def __init__(self, workflows: list[WorkflowBuilder | Callable]):
        """
        Initializes WorkflowBuilder with given attributes.
        """

        self.workflows: list[WorkflowBuilder | Callable] = workflows

    def __call__(self, **kwargs) -> Any:
            workflows: list[WorkflowBuilder | Callable] = self.workflows
            entry_workflow: WorkflowBuilder | Callable = workflows.pop(0)
            result: Any = entry_workflow(**kwargs)

            for workflow in workflows:
                result = workflow(result)

            return result